package info.aspirecn.iov.sjjh.service.impl;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.databind.ObjectMapper;
import info.aspirecn.iov.sjjh.commons.lang.ChannelTypeHandleResponseObject;
import info.aspirecn.iov.sjjh.commons.lang.Constant;
import info.aspirecn.iov.sjjh.config.SupplierConfig;
import info.aspirecn.iov.sjjh.constant.Constants;
import info.aspirecn.iov.sjjh.entity.RequestObject;
import info.aspirecn.iov.sjjh.entity.Response;
import info.aspirecn.iov.sjjh.entity.VehicleHistoryPositionRequestParam;
import info.aspirecn.iov.sjjh.entity.VehicleHistoryPositionResponse;
import info.aspirecn.iov.sjjh.entity.geocoding.GeocodingResponseObject;
import info.aspirecn.iov.sjjh.service.VehicleService;
import info.aspirecn.iov.sjjh.util.DateUtil;
import info.aspirecn.iov.sjjh.util.PostUtil;
import info.aspirecn.iov.sjjh.util.ValidateParamUtil;
import info.aspirecn.rdc.aspirecloud.node.except.utils.ErrorUtils;
import lombok.extern.slf4j.Slf4j;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Slf4j
@Service
public class VehicleServiceImpl implements VehicleService {
    @Autowired
    private SupplierConfig supplierConfig;

    private OkHttpClient okHttpClient = new OkHttpClient.Builder().build();

    @Override
    public ChannelTypeHandleResponseObject vehicleHistoryPositionQuery(HttpServletRequest request, String customBody, int outTime) {
        ChannelTypeHandleResponseObject ret = new ChannelTypeHandleResponseObject();
        ret.setIsCharge(Constant.INTERFACE_QUERY_NO_FEE);
        ret.setCode(Constant.SUCCESS);
        String upstreamCode = "" + Constant.LOG_UPSTREAM_DEFAULT_RESPONSE_CODE;
        request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, upstreamCode);
        if (outTime <= Constants.PARA_ZERO) {
            outTime = Constant.HTTPCLIENT_CONNECTTIMEOUT;
        }

        log.info("supplier-vehicle.vehicleHistoryPositionQuery---customBody={},outTime={}", customBody, outTime);
        // 解析参数
        ObjectMapper objectMapper = new ObjectMapper();
        VehicleHistoryPositionRequestParam requestParam;
        try {
            requestParam = objectMapper.readValue(customBody, VehicleHistoryPositionRequestParam.class);
        } catch (IOException ioe) {
            log.info("supplier-vehicle.vehicleHistoryPositionQuery-解析参数Exception：{}", ioe);
            ErrorUtils.captureException(ioe);
            ret.setCode(Constant.FAIL);
            ret.setResultCode(Constant.INVALID_PARAMETER_CODE);
            ret.setResultBody(Constant.INVALID_PARAMETER);
            ret.setResultDesc(Constant.INVALID_PARAMETER);
            return ret;
        }

        // 参数校验
        if (!ValidateParamUtil.validateRequestParams(requestParam)) {
            ret.setCode(Constant.FAIL);
            ret.setResultCode(Constant.INVALID_PARAMETER_CODE);
            ret.setResultBody(Constant.INVALID_PARAMETER);
            ret.setResultDesc(Constant.INVALID_PARAMETER);
            return ret;
        }

        // 接口调用
        Response response = new Response();
        try {
            RequestObject requestObject = new RequestObject();
            requestObject.setVehicleNumber(requestParam.getPlateNumber());
            requestObject.setLicensePlateTypeCode(requestParam.getPlateColor());
            requestObject.setStartTime(DateUtil.addHours(requestParam.getStartTime(), supplierConfig.getReduceHours()));
            requestObject.setEndTime(DateUtil.addHours(requestParam.getEndTime(), supplierConfig.getAddHours()));
            requestObject.setPartyFunctionCode(supplierConfig.getPartyFunctionCode());
            response = PostUtil.postString(supplierConfig.getHistoryPositionUrl(), JSON.toJSONString(requestObject),
                    supplierConfig.getAppKey(), supplierConfig.getAppSecret(), outTime);
            System.out.println(JSON.toJSONString(response));
        } catch (IOException e) {
            log.info("supplier-vehicle.vehicleHistoryPositionQuery-IOException：{}", e);
            ErrorUtils.captureException(e);
            ret.setCode(Constant.REQUEST_TIMEOUT);
        } catch (Exception e) {
            log.info("supplier-vehicle.vehicleHistoryPositionQuery-Exception：{}", e);
            ErrorUtils.captureException(e);
            ret.setCode(Constant.FAIL);
        }
        upstreamCode = setVehicleHistoryPositionQueryResponse(ret, response, requestParam);
        request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, upstreamCode);
        return ret;
    }

    private String setVehicleHistoryPositionQueryResponse(ChannelTypeHandleResponseObject ret, Response response,
                                                          VehicleHistoryPositionRequestParam requestParam) {
        String upstreamCode = "";
        if (response.getStatusCode() == Constants.STATUS_CODE_200) {
            VehicleHistoryPositionResponse positionResponse = JSON.toJavaObject(JSON.parseObject(response.getBody()),
                    VehicleHistoryPositionResponse.class);
            upstreamCode = positionResponse.getResultCode();
            if (positionResponse.getResultCode().equals(Constants.RESULT_CODE_0)) {
                if (positionResponse.getData() != null && positionResponse.getData().size() > 0) {
                    boolean result = vehicleHistoryPositionQueryByBaidu(positionResponse.getData(), requestParam);
                    if (result) {
                        ret.setResultCode(Constant.MATCH_CODE);
                        ret.setResultBody(positionResponse.getData());
                        ret.setResultDesc(Constant.QUERY_SUCCESS);
                    } else {
                        ret.setResultCode(Constant.UN_MATCH_CODE);
                        ret.setResultBody("核验不一致");
                        ret.setResultDesc(Constant.QUERY_SUCCESS);
                    }
                } else {
                    ret.setResultCode(Constant.UN_FIND_NUM_CODE);
                    ret.setResultBody(Constants.NO_RESULT);
                    ret.setResultDesc(Constants.NO_RESULT);
                }
            } else if (positionResponse.getResultCode().equals(Constants.RESULT_CODE_110000)) {
                ret.setResultCode(Constant.UN_FIND_NUM_CODE);
                ret.setResultBody(Constants.NO_RESULT);
                ret.setResultDesc(Constants.NO_RESULT);
            } else {
                ret.setResultCode(Constant.OTHER_ERROR_CODE);
                ret.setResultBody(Constant.OTHER_ERROR);
                ret.setResultDesc(Constant.OTHER_ERROR);
            }
        } else {
            ret.setResultCode(Constant.OTHER_ERROR_CODE);
            ret.setResultBody(Constant.OTHER_ERROR);
            ret.setResultDesc(Constant.OTHER_ERROR);
            upstreamCode += response.getStatusCode();
        }
        return upstreamCode;
    }

    private boolean vehicleHistoryPositionQueryByBaidu(List<VehicleHistoryPositionResponse.Data> positionList,
                                                    VehicleHistoryPositionRequestParam requestParam) {
        /**
         * 1、云雁调取A-到B+之间所有的车辆轨迹（每30秒有一个经纬度数据）。
         * 2、从A-到A+时间区间中每半个小时抽取一个点位信息的行政区划代码集合，将此集合与A点（起点）行政区划代码进行比对，只要有一点比对一致则视为此点核验通过。
         * 3、从B-到B+时间区间中每半个小时抽取一个点位信息并转化为行政区划代码集合，将此集合与B点（终点）行政区划代码进行比对，只要有一点比对一致则视为此点核验通过。
         * 4、A点到B点时间小于8小时情况依然按次逻辑处理，不做区分。
         * 5、A点和B点都核验通过下视为该条轨迹核验通过，否则为核验不通过。
         * 6、核验不通过情况下返回核验不通过。
         * 7、核验结果一致的情况下输出结果，并输出A点到B点轨迹信息。
         */
        // A-
        String startTimeA = DateUtil.addHours(requestParam.getStartTime(), supplierConfig.getReduceHours());
        // A+
        String endTimeA = DateUtil.addHours(requestParam.getStartTime(), supplierConfig.getAddHours());
        // B-
        String startTimeB = DateUtil.addHours(requestParam.getEndTime(), supplierConfig.getReduceHours());
        // B+
        String endTimeB = DateUtil.addHours(requestParam.getEndTime(), supplierConfig.getAddHours());

        // 4小时*2=8小时（A-A+间隔），8小时*2=16个点（每半个小时取1个点）
        int pointNumber = supplierConfig.getAddHours()*2*2;
        String tempStartTime = startTimeA;
        boolean checkResultA = false;
        // todo 这个集合为了查看位置是否正确
//        List<VehicleHistoryPositionResponse.Data> verificationPositionListA = new ArrayList<>();
//        for (VehicleHistoryPositionResponse.Data position : positionList) {
//            String startTime = tempStartTime;
//            String endTime = DateUtil.addMinutes(startTime, 30);
//            if (Long.parseLong(position.getPositionTime()) > Long.parseLong(startTime)
//                    && Long.parseLong(position.getPositionTime()) <= Long.parseLong(endTime)) {
//                verificationPositionListA.add(position);
//                tempStartTime = endTime;
//                if (getGeocodingCheck(3000,
//                    (position.getLatitudeDegree() + "," + position.getLongitudeDegree()),
//                    requestParam.getLoadingCountrySubdivisionCode())) {
//                    checkResultA = true;
//                }
//            }
//
//        }
//            // todo 这个集合为了查看位置是否正确
//        List<VehicleHistoryPositionResponse.Data> verificationPositionListB = new ArrayList<>();
//        tempStartTime = startTimeB;
//        boolean checkResultB = false;
//        for (VehicleHistoryPositionResponse.Data position : positionList) {
//            String startTime = tempStartTime;
//            String endTime = DateUtil.addMinutes(startTime, 30);
//            if (Long.parseLong(position.getPositionTime()) > Long.parseLong(startTime)
//                    && Long.parseLong(position.getPositionTime()) <= Long.parseLong(endTime)) {
//                verificationPositionListB.add(position);
//                tempStartTime = endTime;
//                if (getGeocodingCheck(3000,
//                        (position.getLatitudeDegree() + "," + position.getLongitudeDegree()),
//                        requestParam.getLoadingCountrySubdivisionCode())) {
//                    checkResultB = true;
//                }
//            }
//
//        }


//        List<String> pointListA = new ArrayList<>();
//        String tempStartTime = startTimeA;
//        pointListA.add(tempStartTime);
//        for (int i = 0; i < pointNumber; i++) {
//            tempStartTime = DateUtil.addMinutes(tempStartTime, 30);
//            pointListA.add(tempStartTime);
//        }
//        List<VehicleHistoryPositionResponse.Data> verificationPositionListA = new ArrayList<>();
//        for (VehicleHistoryPositionResponse.Data position : positionList) {
//            for (int i = 0; i < pointNumber; i++) {
//                Long startTime = Long.parseLong(pointListA.get(i));
//                Long endTime = Long.parseLong(pointListA.get(i+1));
//                Long positionTime = Long.parseLong(position.getPositionTime());
//                if (positionTime >= startTime && positionTime < endTime) {
//                    verificationPositionListA.add(position);
//                    pointListA.remove(i);
//                    break;
//                }
//            }
//            if (verificationPositionListA.size() == pointNumber) {
//                break;
//            }
//        }
//        boolean positionA = false;
//        for (int i = 0; i < verificationPositionListA.size(); i++) {
//            if (getGeocodingCheck(3000,
//                    (verificationPositionListA.get(i).getLatitudeDegree() + "," + verificationPositionListA.get(i).getLongitudeDegree()),
//                    requestParam.getLoadingCountrySubdivisionCode())) {
//                positionA = true;
//                break;
//            }
//        }
//
//        List<String> pointListB = new ArrayList<>();
//        tempStartTime = startTimeB;
//        pointListB.add(tempStartTime);
//        for (int i = 0; i < pointNumber; i++) {
//            tempStartTime = DateUtil.addMinutes(tempStartTime, 30);
//            pointListB.add(tempStartTime);
//        }
//        List<VehicleHistoryPositionResponse.Data> verificationPositionListB = new ArrayList<>();
//        for (int i = 0; i < pointNumber; i++) {
//            for (VehicleHistoryPositionResponse.Data position : positionList) {
//                Long startTime = Long.parseLong(pointListB.get(i));
//                Long endTime = Long.parseLong(pointListB.get(i+1));
//                Long positionTime = Long.parseLong(position.getPositionTime());
//                if (positionTime >= startTime && positionTime < endTime) {
//                    verificationPositionListB.add(position);
//                    break;
//                }
//            }
//            if (verificationPositionListB.size() == pointNumber) {
//                break;
//            }
//        }
//        boolean positionB = false;
//        for (int i = 0; i < verificationPositionListB.size(); i++) {
//            if (getGeocodingCheck(3000,
//                    (verificationPositionListB.get(i).getLatitudeDegree() + "," + verificationPositionListB.get(i).getLongitudeDegree()),
//                    requestParam.getLoadingCountrySubdivisionCode())) {
//                positionB = true;
//                break;
//            }
//        }
//        if (positionA && positionB) {
//            return true;
//        } else {
//            return false;
//        }
        return false;
    }

    /**
     * 校验省区划代码是否一致
     */
    private Boolean getGeocodingCheck(int outTime, String location,String adcode) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            String requestUrl = supplierConfig.getBaiduUrl() + "?ak="+ supplierConfig.getBaiduAk()
                    + "&output=json&coordtype=wgs84ll&location=" + location;
            Request okRequest = new Request.Builder().get().url(requestUrl).build();
            OkHttpClient client = okHttpClient.newBuilder().connectTimeout(outTime, TimeUnit.MILLISECONDS)
                    .readTimeout(outTime, TimeUnit.MILLISECONDS).writeTimeout(outTime, TimeUnit.MILLISECONDS).build();
            okhttp3.Response response = client.newCall(okRequest).execute();
            String responseContext = response.body().string();
            GeocodingResponseObject responseObject = objectMapper.readValue(responseContext,
                    GeocodingResponseObject.class);
            log.info("supplier-vehicle.getGeocodingCheck.responseObject={}", responseContext);
            if (responseObject!=null) {
                if (responseObject.getStatus()==0&&adcode.equals(responseObject.getResult().getAddressComponent().getAdcode())) {
                    return true;
                }
            }
            return false;
        } catch (Exception e) {
            ErrorUtils.captureException(e);
            log.info("supplier-vehicle.Exception.e={}", e);
        }
        return false;
    }
}
